{TGlNestingInfo v1.1 copyright Greg Lorriman 1998 nestinfo@lorriman.demon.co.uk}

{set font to Courier New, size 8}

{Date 11 Feb 1998, initial release 11 Feb 1998}

{RXLIB SUPPORT}
{if RxLib is present then add $ to define on next line, ie $define....}
{DO NOT remove braces, see examples directly below}

{define rxLibinGL}

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$DEFINE GL_D3} { Delphi 3.0 or higher with Tsplitter control}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

{Declares and defines the TGlNestingInfo component}
unit Nestinfo;

interface

uses forms,controls, classes, extctrls,sysutils{$ifdef rxLibinGL},
	 rxsplit{$endif};

type
	DefglNiniOptions = (niType,niAlignment,niSiblings,niRxInfo);
	TglNiniOptions=set of DefglNiniOptions;
	TNiSplitterOptions=(niInclude,niExclude);
	TglNiView=(niPanelling,niAll);

type
{Design time component that shows the nesting of controls
of the form on which it is placed. Special support for panels, splitters and the RxLib splitter.

Facilities for runtime usage (see Refresh).

Usage : Place on form and click on 'Nesting' in the object inspector. 

Output can be modified by changing various properties of the component in the object inspector.}
TGlNestingInfo=class(TComponent)
   private

   	FstrList : Tstringlist;
       Foptions : TglNiniOptions;
       Findent : string;
       FsplitterOpts : TNiSplitterOptions;
       FView : TglNiView;
       FForm : TForm;

       siblingNumber : integer;

   protected

       function getIndent:integer;virtual;
       procedure setIndent(i : integer);virtual;
       function createDepthedString(s : string; i : integer):string;virtual;
       function hasPanels(p : Twincontrol):boolean;virtual;

   	function isSplitter( s : Tcontrol):boolean;virtual;
       procedure setstrings(s : Tstringlist);virtual;
       function readStrings:Tstringlist;virtual;
       procedure setOptions(o : TglNiniOptions);
       function getPanelInfo(f : Tform; s : Tstringlist):Tstringlist;virtual;
       function getControlInfo(f : Tform; s : Tstringlist):Tstringlist;virtual;
       procedure processPanel(p : twincontrol;strlist : tstringlist;
       	depth : integer); virtual;
       procedure processWincontrol(p : twincontrol;strlist : tstringlist;
       	depth : integer); virtual;
       procedure processControl(p : TWinControl;strlist : tstringlist;
       	depth : integer); virtual;
       function createString(depth : integer;siblingString,name,alignString,
       	typeString: string):string;virtual;

   public

		constructor Create(AOwner: TComponent); override;
		destructor destroy;override;
       {For runtime use of component. Re-indexes controls ready for output via Nesting property}
		procedure refresh;
       {The default form processed is the form on which the component is placed.

       Use this property at runtime to get nesting info about another form. Useful for
       documenting projects. Use with Refresh.}
		property Form : TForm read FForm Write FForm ;

  published
	{Click on this property in the object inspector to get nesting info.}
  		property Nesting : Tstringlist  read readStrings write setstrings
       	stored false ;

       {
       Various options that affect output:

       niType, include type info

       niAlignment, include alignment info

       niSiblings, numbers controls in list to indicate sibling relationship. convenient in case of large numbers of controls

       niRxInfo, includes extra RxSplitter property info (controlFirst/ControlSecond).}
       property Options : TglNiniOptions read Foptions write setOptions;

       {Sets the indent used for representation of the nesting.}
       property Indent : integer read getIndent write setIndent;

       {Include or exclude splitters. Splitters are detected by type comparison in the
       case of RxSPlitter and the Delphi3 Splitter control, or by looking for the string 'split' in the type name.
       you will need to switch this property on if a control being used is not a splitter but includes the string 'split'
       in its type name}
       property Splitters : TNiSplitterOptions read Fsplitteropts
       	write Fsplitteropts;

       {Allows display of panel controls only. Includes all controls derived from TCustomPanel
        (which includes some third party splitters).

       If niPanelling is chosen then non-panel controls that contain sub-panels are still displayed but
       within angle brackets.}
       property View : TglNiView read FView write FView;

end;


procedure Register;



implementation

const
glNiAlignment: array[alNone..alClient] of string=(
'alNone','alTop','alBottom','alLeft', 'alRight', 'alClient');

const
minIndent=1;
maxIndent=10;

function getAlignName(a : Talign):string;forward;

{recursive function}
procedure TGlNestingInfo.processPanel(p : twincontrol; strlist : Tstringlist;
	depth : integer);
var
	cntr, siblingNum : integer;
   rxInfoString,  typeString,
   siblingString, AlignString : string;
begin
	inc(depth);

   {following 4 lines ensure that sibling numbering increments properly}
   if (p.controlcount=0) then
	   exit;
   if hasPanels(p) then
	   inc(siblingNumber);

   siblingNum:=siblingNumber;

  	for cntr:=0 to p.controlcount-1 do begin

       if p.controls[cntr]=nil then
           continue;
       if not (p.controls[cntr] is Twincontrol) then
	           continue;

       rxInfoString:='';
       typeString:='';
       siblingString:='';
       AlignString:='';


       {$ifdef rxLibinGL} {for rxlib splitter}
       if p.controls[cntr] is TrxSplitter then begin
       	if FSplitterOpts=niExclude then continue;
           with p.controls[cntr] as TrxSplitter do begin

               if niRxInfo in Foptions then begin
                   if controlfirst<>nil then
                       rxInfoString:='  {1st:'+controlfirst.name+'}';
                   if controlsecond<>nil then
                       rxInfoString:='  {2nd:'+controlsecond.name+'}';
               end;

               if niAlignment in FOptions then
                   alignString:=getAlignName(align);
               if niType in Foptions then
                   typeString:=className;
               if niSiblings in Foptions then
                   siblingString:=intToStr(siblingNum);

               strlist.add(createString(depth,siblingString,name,alignString,
               	typeString)+rxInfoString);
           end;{with}
           processpanel(twincontrol(p.controls[cntr]),strlist,depth);
           continue;
       end;
       {$endif}

       {$ifdef gl_d3} {For delphi3 Tsplitter}
       if p.controls[cntr] is TSplitter then begin
         	if  FSplitterOpts=niExclude then continue;
           with p.controls[cntr] as TSplitter do begin

               if niAlignment in FOptions then
                   alignString:=getAlignName(align);
               if niType in Foptions then
                   typeString:=className;
               if niSiblings in Foptions then
                   siblingString:=intToStr(siblingNum);

               Fstrlist.add(createString(depth,siblingString,name,alignString,
               	typeString));
           end;{with}
           processpanel(twincontrol(p.controls[cntr]),strlist,depth);
           continue;
       end;
       {$endif}

       {for custompanels with "split" in the classname}
       if (p.controls[cntr] is tcustompanel) and
       	(pos('split',lowercase(p.controls[cntr].classname))>0) then begin
       	if  FSplitterOpts=niExclude then continue;
           with p.controls[cntr] as Tcustompanel do begin

               if niAlignment in FOptions then
                   alignString:=getAlignName(align);
               if niType in Foptions then
                   typeString:=className;
               if niSiblings in Foptions then
                   siblingString:=intToStr(siblingNum);

               strlist.add(createString(depth,siblingString,name,alignString,
               	typeString));
           end;{with}
           processpanel(twincontrol(p.controls[cntr]),strlist,depth);
           continue;
       end;



       {basic panel}
		if (p.controls[cntr] is tpanel) and
       	(p.controls[cntr].classname='TPanel') then begin
          	with p.controls[cntr] as Tpanel do begin

               if niAlignment in FOptions then
                   alignString:=getAlignName(align);
               if niSiblings in Foptions then
                   siblingString:=intToStr(siblingNum);
               {UNCOMMENT FOR niType CONSISTENCY}
               {if niType in Foptions then
                   typeString:=className;}

	       		strlist.add(createString(depth,siblingString,name,alignString,
               	typeString));
           end;{with}
           processpanel(twincontrol(p.controls[cntr]),strlist,depth);
           continue;
       end;

      {for custompanels, reason why this comes last is that splitters are also
      custompanels}
		if p.controls[cntr] is tcustompanel then begin
          	with p.controls[cntr] as Tcustompanel do begin

                   if niAlignment in FOptions then
                   	alignString:=getAlignName(align);
                   if niType in Foptions then
                   	typeString:=className;
                   if niSiblings in Foptions then
                   	siblingString:=intToStr(siblingNum);

					strlist.add(createString(depth,siblingString,name,
                   	alignString,typeString));
           end;{with}
           processpanel(twincontrol(p.controls[cntr]),strlist,depth);
           continue;
       end;

       {if is not TCustomPanel or Tsplitter based then it may still contain
       something}
       if p.controls[cntr] is Twincontrol then begin
       	processWinControl(twincontrol(p.controls[cntr]),Fstrlist,depth);
       end;

   end; {for}
end;

{for non-TcustomPanel based controls derived from Twincontrol}
procedure TGlNestingInfo.processWinControl( p :  twincontrol;strlist : tstringlist;
       	depth : integer);
var cntr : integer;
begin
	if p.controlcount=0 then
   	exit;

	if hasPanels(p) then begin
      FstrList.add(createDepthedString(Findent,depth)+'<'+p.name+'>');
      processpanel(p,FstrList,depth);
   end
   else begin
   	for cntr:=0 to p.controlCount-1 do begin
       	if p.controls[cntr]=nil then continue;
       	if p.controls[cntr] is TWinControl then
           	processWinControl(TwinControl(p.controls[cntr]),FstrList,depth);
       end;
   end;

end;

function TGlNestingInfo.createDepthedString(s : string; i : integer):string;
var cntr : integer;
str : string;
begin
	str:='';
	for cntr:=0 to i-1 do
   	str:=str+s;
   result:=str;
end;

{glNiAlignment in function for clarity only}
function getAlignName(a : Talign):string;
begin
	result:=glNiAlignment[a];
end;

function TGlNestingInfo.getPanelInfo(f : Tform; s : Tstringlist):Tstringlist;
begin
	processpanel(twincontrol(f),s,0);
   result:=s;
end;

function TGlNestingInfo.getControlInfo(f : Tform; s : Tstringlist):Tstringlist;
begin
	processcontrol(TWinControl(f),s,0);
   result:=s;
end;


constructor TGlNestingInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FstrList:=TstringList.create;
  Foptions:=[niAlignment,niSiblings{$ifdef gldebug},niType{$endif}];
  FView:=niAll;
  FSplitterOpts:=niInclude;
  Findent:='    ';
  fform:=TForm(AOwner);

end;


function TGlNestingInfo.readStrings:Tstringlist;
begin
	result:=FstrList;
   if (csDesigning in ComponentState) or (Fstrlist.count=0) then
       refresh;

end;


procedure TGlNestingInfo.refresh;
begin

	FstrList.clear;
   siblingNumber:=0;

  {classname, instead of name, due to D1 bug}
  FstrList.add(Tform(Fform).classname);

	if View=niPanelling then
		getPanelInfo(TForm(Fform),FstrList)
	else
		getcontrolInfo(TForm(Fform),FstrList);
end;

procedure TGlNestingInfo.setstrings(s : Tstringlist);
begin
{do nothing}
   if not (csDesigning in ComponentState) then
   	raise exception.create('Cannot assign to this property');

end;

{The format of the output is determined by this function}
function TGlNestingInfo.createString(depth : integer;siblingString,name,
	alignString,typeString:string):string;
var
	p,a,t : string;

begin

	p:='';
	a:='';
   t:='';

   if siblingString<>'' then
   	p:=siblingString+': ';
   if alignString<>'' then
   	a:='  {'+alignString+'}';
   if typeString<>'' then
   	t:='  {'+typeString+'}';

	result:=createDepthedString(Findent,depth)+p+name+a+t;
end;


function TGlNestingInfo.getIndent:integer;
begin
	result:=length(Findent);
end;

procedure TGlNestingInfo.setIndent(i : integer);
var
	str : string;
   cntr : integer;
begin
	if i<minIndent then i:=minIndent;
   if i>maxIndent then i:=maxIndent;

	str:='';
   for cntr:=0 to i-1 do begin
   	str:=str+' ';
   end;
   Findent:=str;
end;

function TGlNestingInfo.hasPanels(p : Twincontrol):boolean;
var
	cntr : integer;
begin
   result:=false;
	for cntr:=0 to p.controlcount-1 do begin
   	if p.controls[cntr]=nil then
       	continue;
   	if p.controls[cntr] is TcustomPanel then begin
       	result:=true;
           exit;
       end;
   end;
end;


{recursive function}
procedure TGlNestingInfo.processcontrol(p : TwinControl; strlist : Tstringlist;
	depth : integer);
var
	cntr, siblingNum : integer;
  typeString,
   siblingString, AlignString, rxInfoString : string;
begin
	inc(depth);

   {following 4 lines ensure that sibling numbering increments properly}
   if p is Twincontrol then
	   if (p.controlcount=0) then
		   exit
      else
		   inc(siblingNumber);

   siblingNum:=siblingNumber;

  	for cntr:=0 to p.controlcount-1 do begin

       if p.controls[cntr]=nil then
           continue;
       if not (p.controls[cntr] is Tcontrol) then
	           continue;

       typeString:='';
       siblingString:='';
       AlignString:='';

       if p.controls[cntr] is Tcontrol then begin

           if isSplitter(p.controls[cntr]) then  begin
           	if splitters=niExclude then
               	continue;
               {$ifdef rxLibinGL} {for rxlib splitter}
               if p.controls[cntr] is TrxSplitter then begin
                   if FSplitterOpts=niExclude then continue;
                   with p.controls[cntr] as TrxSplitter do begin

                       if niRxInfo in Foptions then begin
                           if controlfirst<>nil then
                               rxInfoString:='  {1st:'+controlfirst.name+'}';
                           if controlsecond<>nil then
                               rxInfoString:='  {2nd:'+controlsecond.name+'}';
                       end;

                       if niAlignment in FOptions then
                           alignString:=getAlignName(align);
                       if niType in Foptions then
                           typeString:=className;
                       if niSiblings in Foptions then
                           siblingString:=intToStr(siblingNum);

                       strlist.add(createString(depth,siblingString,name,alignString,
                           typeString)+rxInfoString);
                   end;{with}
                   processpanel(twincontrol(p.controls[cntr]),strlist,depth);
                   continue;
               end;
               {$endif}
	       end;


           with p.controls[cntr] as Tcontrol do begin

               if niAlignment in FOptions then
               	if p.controls[cntr] is Twincontrol then
	                   alignString:=getAlignName(align);
               if niType in Foptions then
                   typeString:=className;
               if niSiblings in Foptions then
                   siblingString:=intToStr(siblingNum);

               strlist.add(createString(depth,siblingString,name,alignString,
               	typeString)+'');
           end;{with}
           if p.controls[cntr] is TWinControl then
	           processcontrol(TWinControl(p.controls[cntr]),strlist,depth);
           continue;
       end;
   end;
end;







destructor TGlNestingInfo.destroy;
begin

	FstrList.free;
   FstrList:=nil;

	inherited destroy;
end;

function TGlNestingInfo.isSplitter( s : Tcontrol):boolean;
begin
	result:=false;
   {$ifdef rxLibinGL} {for rxlib splitter}
  if s is TrxSplitter then
       result:=true;
  {$endif}
   {$ifdef gl_d3} {For delphi3 Tsplitter}
   if s is TSplitter then
       result:=true;
   {$endif}
   if (s is tcustompanel) and
       (pos('split',lowercase(s.classname))>0) then
       result:=true;
end;

procedure TGlNestingInfo.setOptions(o : TglNiniOptions);
begin
	Foptions:=o;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TGlNestingInfo]);
end;

end.

